



gene.ctree3<-function(data, covar, thr.sample, thr.grp.sample, thr.leaf.size, nleaf, 
                cut.point, nperm, backward = TRUE, trace = FALSE, seed=0, two.way = FALSE, 
                indicator2 = NULL, cal.pval = TRUE, path.op = NULL, path.op.back = NULL, 
                debug=FALSE, bin=TRUE){
  
  if(two.way){
    two.way<-1
    if(is.null(indicator2)){
      stop("Error: indicator2 cannot be NULL if two_way is TRUE")
    }
  }else{
    two.way<-0
    indicator2<-data[,-1,drop=F]
  }
  
  if(backward){
    backward<-1
  }else{
    backward<-0
  }
  
  if(cal.pval){
    cal.pval<-1
    path.op<-"no_output"
    path.op.back<-"no_output"
  }else{
    cal.pval<-0
    if(is.null(path.op)){
      stop("Error: path.op cannot be NULL if cal.pval is FALSE")
    }
    
    if(is.null(path.op.back) && backward){
      stop("Error: path.op.back cannot be NULL if cal.pval is FALSE and backward is TRUE")
    }
  }
  
  data<-data[order(covar), ,drop=F]
  indicator2<-indicator2[order(covar), ,drop=F]
  
  covar<-sort(covar)
  
  nind<-ncol(data)-1
  nind2<-ncol(indicator2)
  ngrp<-length(unique(covar))
  N<-nrow(data)
  data<-as.vector(as.matrix(data))
  indicator2<-as.vector(as.matrix(indicator2))
  
  if(is.null(cut.point)){
    cut.point<-2:nleaf
  }else{
    cut.point<-sort(cut.point[cut.point<=nleaf & cut.point>=2])
  }
  ncp<-length(cut.point)
  nleaf<-min(nleaf, max(cut.point))
  
  pval<-rep(-1, ncp+1)
  pval.back<-rep(-1, ncp+1)
  
  if(is.null(seed)){
    seed = -1
  }
  
  if(trace){
    trace<-1
  }else{
    trace<-0
  }
  
  if(is.null(debug)){
    debug<-0
  }
  
  if(debug){
    debug<-1
  }else{
    debug<-0
  }
  if(bin){
    bin<-1
  }else{
    bin<-0
  }
  
  tree.model<-rep(0, 6*(2*nleaf-3))
  
  
  
  #dyn.load("/data/zhangh12/multinomial/code/gene_ctree/gene_ctree3.so")
  tmp<-.C("gene_ctree3", as.integer(data), as.integer(indicator2), as.integer(covar), 
        as.integer(two.way), as.integer(backward), as.integer(cal.pval), 
        as.character(path.op), as.character(path.op.back), as.integer(N), 
        as.integer(ngrp), as.integer(nind), as.integer(nind2), as.integer(thr.sample), 
        as.integer(thr.grp.sample), as.integer(thr.leaf.size), as.integer(nleaf), 
        as.integer(cut.point), as.integer(ncp), as.integer(nperm), as.integer(trace), 
        as.integer(seed), as.integer(debug), as.integer(bin), pval=as.double(pval), 
        pval.back=as.double(pval.back), tree.model=as.integer(tree.model))
  #dyn.unload("/data/zhangh12/multinomial/code/gene_ctree/gene_ctree3.so")
 
  adj.pval<-tmp$pval[1]
  unadj.pval<-tmp$pval[-1]
  tree.model=as.data.frame(matrix(tmp$tree.model,ncol=6,byrow=T))
  tree.model.forward<-tree.model[tree.model[,1]==0,-1]
  tree.model.backward<-tree.model[tree.model[,1]==1,-(1:3)]
  colnames(tree.model.forward)<-c("1.way", "indi.ID", "parent", "child1", "child2")
  tree.model.forward$"1.way"<-ifelse(tree.model.forward$"1.way"==1, TRUE, FALSE)
  colnames(tree.model.backward)<-c("parent", "child1", "child2")
  
  if(adj.pval*(nperm+1)<10){
    comment<-"refine"
  }else{
    comment<-"done"
  }
  
  adj.pval.back<-tmp$pval.back[1]
  unadj.pval.back<-tmp$pval.back[-1]
  
  if(adj.pval.back*(nperm+1)<10){
    comment.back<-"refine"
  }else{
    comment.back<-"done"
  }
  
  
  list(adj.pval= adj.pval, unadj.pval=unadj.pval, comment=comment, 
     adj.pval.back=adj.pval.back, unadj.pval.back=unadj.pval.back, 
     comment.back=comment.back, tree.model.forward = tree.model.forward, 
     tree.model.backward = tree.model.backward, cut.point=cut.point)
  
}



##Dear Bill,
##
##Here are updated directions about the interface of the package we want.
##Basically we want the user to provide the outcome (0/1), 
# covariates and the standard genotypes data. 
# But our main function, i.e. gene.ctree3() above, requires some special input. 
# Next I will introduce how to prepare the three major arguments in gene.ctree3() 
# from outcome, covariates and genotypes provided by the user.
##
##1. data. It is a n X (p + 1) data.frame in which the first column is the outcome. 
# n is the total sample size. The outcome must be encoded as 0/1. 
# The other p columns are also encoded as 0/1, and they are converted from the genotypes.
##The missing values in genotype must be imputed by the user. 
# Then for every SNP, define the following two vectors, v1 and v2, so that
##               v1 = 1 if SNP == 0 or 1
##               v1 = 0 if SNP == 2
##               v2 = 1 if SNP == 1 or 2
##               v2 = 0 if SNP == 0
##If we have K SNPs, then the total number of such vectors are p = 2K, and data is a 
# n X (2K + 1) data.frame. The name of the columns are not essential for calling gene.ctree3()
##
##2. covar. It is a ONE-dimensional integer vector with length n. 
# We require the user uses categorical covariates only. 
# And the multiple categorical covariates must be converted into ONE-dimensional vector "covar". 
# For example, one original covariate is Age Group with 3 levels, and the other original 
# covariate is treatment with 2 levels, then we want to convert them into "covar" with 
# 3 X 2 = 6 levels. I implement this as follows:
##               covar<-as.integer(as.factor(paste(age, treat, sep="+")))-1
##Ideally, the user can specify the columns of covariates in a formula manner, 
# and our interface can identify those columns from the input data and convert them to "covar" 
# for the user. 
# Please note that the function gene.ctree3() requires "covar" is a integer vector, 
# and its elements have values from 0 to length(unique(covar))-1. 
# I would like to add warning in the manual that too many original covariates will result 
# in too many levels in "covar", which will reduce the statistical power. 
# I would like to suggest the user to select the most relevant covariates by themself 
# before using our test. 
# In addition, the interface for "covar" in a formula manner is optional if it is too 
# complicated to implement. I really appreciate your help.
##
##3. The function gene.ctree3() also requires several other inputs. 
# Some are integers, e.g. thr.sample, thr.grp.sample, thr.leaf.size, nleaf, nperm. 
# Some are bool value, e.g. two.way, backward, cal.pval, etc. 
# In addition, cut.point is an integer vector. You can place these arguments in the interface. 
# Bellow are the description of these arguments, which you can place in the manual
##
##(a) thr.sample. Integer. The minimum sample size for a node to be considered splitted. 
# If the sample size in a node is too samll, we don't further split it. Default 50
##(b) thr.grp.sample. INteger. Recall that "covar" is a L-level factor which defines the 
# group of the samples. 
# Our test requires that the sample size corresponding to the l-th level in a node to be 
# splitted is not too small. This argument defines the lower limit. Default 2
##(c) thr.leaf.size. Integer. The minimum sample size for a leaf node in the tree. 
# Set its default as 20
##(d) nleaf. Integer. The maximum number of leaves in a tree. Default 5
##(e) cut.point. Integer vector. Default NULL
##(f) nperm. Integer. Number of permutation steps for computing p-value. Default 10000
##(g) backward. Bool. If TRUE, computing the p-value with backward searching as well. Default TRUE
##(h) trace. Bool. If TRUE, print some details when the tree is growing. Default FALSE
##(i) seed. Integer. Seed for random number generator. 
# If negative, the program will use the system time as the seed. Default 0
##
##(j) Please mute the following arguments from the interface (and set them as follows):
##  (j.1) two.way = debug = FALSE
##  (j.2) cal.pval = bin = TRUE
##  (j.3) path.op = path.op.back = indicator2 = NULL
##
##
##So the interface we want is like this:
##foo(formula, data, op = list(thr.sample, thr.grp.sample, ..., cut.point))
##where data includes outcome, genotype and original covariates
##Or a simplier interface if the one above is too complicated to implement
##foo(outcome, genotypes, covariates, op = ...)
##




