
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include <string.h>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <vector>
#include <math.h>
#include <algorithm>
#include <string>
#include <assert.h>
#include <float.h>
#include "matrix.hpp"


using namespace std;

typedef unsigned long long uint64;
typedef vector<uint64> bitvec;
typedef vector<vector<int> > Table;

struct SCORE{
	
	double score;
	int id;
	
	bool operator < (const SCORE& sc) const{
		return (score < sc.score);
	}
	
	SCORE(double s, int i){
		score = s;
		id = i;
	}
	
};

SCORE score0(.0, -1);

typedef vector<SCORE> VecScore;

struct NODE{
	
	bitvec sample_index; // indicate that which samples are belong to the node
	vector<bool> valid_indicator; // indicate that which snp indicators should be accessed for splitting
	vector<int> grp_ncase; // indicate that the case number in each group defined by covariate
	vector<int> grp_nctrl; // indicate that the control number in each group defined by covariate
	vector<int> grp_N;
	vector<Table> cnt; // 2 x 2 contingency table of snp indicator in each group. For temporary use
	vector<Table> candidate_split_cnt; // 2 x 2 contingency table of the candidate snp indicator in each group
	int node_id; // id of the node
	int split_index; // the index of the snp indicator finally chosen to split the node. It is set only if the node is not a terminal
	int candidate_split_index; // the index of the best condidate of snp indicator in the node
	int ncase; // number of case in the node
	int nctrl; // number of control in the node
	int N; // total sample size in the node
	int min_group_sample_size; // min grp_ncase+grp_nctrl
	bool processed; // indicate that whether the node is needed to be process 
	bool terminal; // indicate that whether the node is set as terminal
	double candidate_split_stat; // the largest statistics if splitted
	
	NODE *parent;
	NODE *child1;
	NODE *child2;
	
	NODE(const bitvec &input_sample_index, const vector<bool> &input_valid_indicator, 
	vector<int> input_grp_ncase, vector<int> input_grp_nctrl, 
	int input_node_id, int input_split_idx, int input_candidate_split_index, 
	bool input_processed, bool input_terminal, double input_candidate_split_stat, 
	NODE *const par, NODE *const chd1, NODE *const chd2){
		sample_index = input_sample_index;
		valid_indicator = input_valid_indicator;
		node_id = input_node_id;
		split_index = input_split_idx;
		candidate_split_index = input_candidate_split_index;
		grp_ncase = input_grp_ncase;
		grp_nctrl = input_grp_nctrl;
		int ngrp = grp_ncase.size();
		grp_N = vector<int>(ngrp, 0);
		ncase = 0;
		nctrl = 0;
		for(int l = 0; l < ngrp; ++l){
			ncase += grp_ncase[l];
			nctrl += grp_nctrl[l];
			grp_N[l] += grp_nctrl[l] + grp_ncase[l];
		}
		N = ncase + nctrl;
		processed = input_processed;
		terminal = input_terminal;
		candidate_split_stat = input_candidate_split_stat;
		cnt = vector<Table> (ngrp, Table(3, vector<int> (3, 0)));
		candidate_split_cnt = vector<Table> (ngrp, Table(2, vector<int> (2, 0)));
		min_group_sample_size = 10000000;
		for(int l = 0; l < ngrp; ++l){
			cnt[l][0][2] = grp_nctrl[l];
			cnt[l][1][2] = grp_ncase[l];
			cnt[l][2][2] = cnt[l][0][2] + cnt[l][1][2];
			if(cnt[l][2][2] < min_group_sample_size){
				min_group_sample_size = cnt[l][2][2];
			}
		}
		
		parent = par;
		child1 = chd1;
		child2 = chd2;
	}
	
	NODE(NODE &node){
		grp_ncase = node.grp_ncase;
		grp_nctrl = node.grp_nctrl;
		grp_N = node.grp_N;
		N = node.N;
		
		parent = NULL;
		child1 = NULL;
		child2 = NULL;
	}
	
};


unsigned char wordbits[65536];
/*
inline uint64_t rdtsc() {
    uint32_t lo, hi;
    __asm__ __volatile__ (
      "xorl %%eax, %%eax\n"
      "cpuid\n"
      "rdtsc\n"
      : "=a" (lo), "=d" (hi)
      :
      : "%ebx", "%ecx");
    return (uint64_t)hi << 32 | lo;
}
*/
inline int bitcount(uint64 x){

	x -= (x >> 1) & 0x5555555555555555;
	x = (x & 0x3333333333333333) + ((x >> 2) & 0x3333333333333333);
	x = (x + (x >> 4)) & 0x0f0f0f0f0f0f0f0f;
	x += x >> 8;
	x += x >> 16;
	x += x >> 32;
	return x & 0x7f;

}

inline bitvec operator&(const bitvec& bv1, const bitvec& bv2){

	bitvec bv(bv1.size(), 0);

	for(int i = 0; i < bv1.size(); ++i){
		bv[i] = bv1[i] & bv2[i];
	}

	return bv;

}


ostream& operator<<(ostream &os, const vector<double> &v){
	
	for(int i = 0; i < v.size(); ++i){
		os << v[i] << endl;
	}
	
	return os;
	
}



